static const char *RcsId = "$Header:  $";
//+=============================================================================
//
// file :         ScriptRunner.cpp
//
// description :  C++ source for the ScriptRunner and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                ScriptRunner are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author:  $
//
// $Revision:  $
//
// $Log:  $
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name|  Method name
//	----------------------------------------
//  State      |  dev_state()
//  Status     |  dev_status()
//  RunScript  |  run_script()
//
//===================================================================

//#include <botan/botan.h>

#include <tango.h>
#include <ScriptRunner.h>
#include <ScriptRunnerClass.h>

#include <unistd.h>
#include <sys/wait.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/types.h>

//#include "botan_utils.h"

using namespace std;

string my_basename(string in)
{
	char * in_cstr = strdup(in.c_str());
	char * out_cstr = basename(in_cstr);
	string out(out_cstr);
	free(in_cstr);
	
	return out;
}

namespace ScriptRunner_ns
{

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::ScriptRunner(string &s)
// 
// description : 	constructor for simulated ScriptRunner
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
ScriptRunner::ScriptRunner(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

ScriptRunner::ScriptRunner(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

ScriptRunner::ScriptRunner(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void ScriptRunner::delete_device()
{
	//	Delete device's allocated object
	if (thread_valid)
		pthread_join(thread, NULL);
	thread_valid = false;

	//	Delete device's allocated object
	pthread_mutex_destroy(&output_values_mutex);
	pthread_mutex_destroy(&state_mutex);

	//delete botan_utils;
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void ScriptRunner::init_device()
{
	INFO_STREAM << "ScriptRunner::ScriptRunner() create device " << device_name << endl;

	thread_valid = false;
	encryption = false;
	
	pthread_mutexattr_t mutexattr;
	pthread_mutexattr_init(&mutexattr);
	pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
	pthread_mutex_init(&output_values_mutex, &mutexattr);
	pthread_mutex_init(&state_mutex, &mutexattr);
	pthread_mutexattr_destroy(&mutexattr);
	
	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	
	scriptsRepositoryPath = defaultScriptsRepositoryPath;
	exitStatus = -1;
	attr_ExitStatus_read = &exitStatus;

	// set up encryption if requested
	
	//botan_utils = NULL;
	
	/*
	if (encryption == false)
	{
		// no encryption requested, leave botan_utils to NULL, this will disable encryption
	}
	else
	{
		if (devicePrivateKeyFile.empty() || clientPublicKeyFile.empty())
		{
			TangoSys_OMemStream os;
			os << "Error, encryption requested but properties DevicePrivateKeyFile and/or ClientPublicKeyFile are empty" ;
			Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
		}
		
		bool has_passphrase = !devicePrivateKeyPassphraseFile.empty();
	
		// create encryption utility object
		
		botan_utils = new BotanUtils(devicePrivateKeyFile, clientPublicKeyFile, has_passphrase, devicePrivateKeyPassphraseFile);
		if (!botan_utils->init_ok())
		{
			Tango::Except::throw_exception("Illegal value", botan_utils->init_message(), __func__); 
		}
	}
	*/	

	// set up access control on ip addresses if requested
	
	if (allowedIPFile.empty())
	{
		access_control_on = false;
		DEBUG_STREAM << "Access control on IP addresses not enabled." << endl;
	}
	else
	{
		access_control_on = true;
		
		ifstream myfile (allowedIPFile.c_str());
		if (myfile.is_open())
		{
			while (!myfile.eof())
			{
				string line;
				getline (myfile,line);
				
				get_ips_from_host(line, allowed_ips);
			}
			myfile.close();
		}
		else
		{
			DEBUG_STREAM << "Access control on IP addresses enabled, but could not load file \""<< allowedIPFile.c_str() << "\". No client will be allowd to connect!! Error: " << strerror(errno) << endl;
		}
		
		// diagnostics
		
		if (allowed_ips.empty())
		{
			DEBUG_STREAM << "Access control on IP addresses enabled, but file \""<< allowedIPFile.c_str() << "\" was empty. No client will be allowd to connect!!" << endl;
		}
		else
		{
			DEBUG_STREAM << "Access control on IP addresses enabled." << endl;
			
			DEBUG_STREAM << "Allowed IPs:"; 
			for (std::set<std::string>::const_iterator it = allowed_ips.begin(); it != allowed_ips.end(); ++it )
			{
				DEBUG_STREAM << *it; 
			}
			DEBUG_STREAM << "End of allowed IPs."; 
		}
	}
		

	
	safe_set_state_status(Tango::ON, "Idle");
}


//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void ScriptRunner::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("DefaultScriptsRepositoryPath"));
	dev_prop.push_back(Tango::DbDatum("Encryption"));
	dev_prop.push_back(Tango::DbDatum("DevicePrivateKeyFile"));
	dev_prop.push_back(Tango::DbDatum("DevicePrivateKeyPassphraseFile"));
	dev_prop.push_back(Tango::DbDatum("ClientPublicKeyFile"));
	dev_prop.push_back(Tango::DbDatum("AllowedIPFile"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	ScriptRunnerClass	*ds_class =
		(static_cast<ScriptRunnerClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize DefaultScriptsRepositoryPath from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultScriptsRepositoryPath;
	else {
		//	Try to initialize DefaultScriptsRepositoryPath from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultScriptsRepositoryPath;
	}
	//	And try to extract DefaultScriptsRepositoryPath value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultScriptsRepositoryPath;

	//	Try to initialize Encryption from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  encryption;
	else {
		//	Try to initialize Encryption from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  encryption;
	}
	//	And try to extract Encryption value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  encryption;

	//	Try to initialize DevicePrivateKeyFile from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  devicePrivateKeyFile;
	else {
		//	Try to initialize DevicePrivateKeyFile from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  devicePrivateKeyFile;
	}
	//	And try to extract DevicePrivateKeyFile value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  devicePrivateKeyFile;

	//	Try to initialize DevicePrivateKeyPassphraseFile from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  devicePrivateKeyPassphraseFile;
	else {
		//	Try to initialize DevicePrivateKeyPassphraseFile from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  devicePrivateKeyPassphraseFile;
	}
	//	And try to extract DevicePrivateKeyPassphraseFile value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  devicePrivateKeyPassphraseFile;

	//	Try to initialize ClientPublicKeyFile from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  clientPublicKeyFile;
	else {
		//	Try to initialize ClientPublicKeyFile from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  clientPublicKeyFile;
	}
	//	And try to extract ClientPublicKeyFile value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  clientPublicKeyFile;

	//	Try to initialize AllowedIPFile from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  allowedIPFile;
	else {
		//	Try to initialize AllowedIPFile from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  allowedIPFile;
	}
	//	And try to extract AllowedIPFile value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  allowedIPFile;



	//	End of Automatic code generation
	//------------------------------------------------------------------

        i=-1;
	bool updateDB = false;
        /// The current properties are:
        ///
	if (dev_prop[++i].is_empty()) {
	        dev_prop[i] << defaultScriptsRepositoryPath;
		updateDB = true;
	}
	if (dev_prop[++i].is_empty()) {
	        dev_prop[i] << encryption;
		updateDB = true;
	}
	if (dev_prop[++i].is_empty()) {
	        dev_prop[i] << devicePrivateKeyFile;
		updateDB = true;
	}
	if (dev_prop[++i].is_empty()) {
        	dev_prop[i] << devicePrivateKeyPassphraseFile;
		updateDB = true;
	}
	if (dev_prop[++i].is_empty()) {
		dev_prop[i] << clientPublicKeyFile;
		updateDB = true;
	}
	if (dev_prop[++i].is_empty()) {
		dev_prop[i] << allowedIPFile;
		updateDB = true;
	}

	if ((Tango::Util::instance()->_UseDb==true) && (updateDB))
		get_db_device()->put_property(dev_prop);

}
//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void ScriptRunner::always_executed_hook()
{
	if (!access_control_on)
		return;
	
	string clientAddress;
	
	Tango::DevVarStringArray blackbox = black_box(1)[0];
	string last_msg(blackbox[0].in());
	
	string::size_type loc = last_msg.find( " requested from ", 0 );
	if( loc != string::npos )
   	{
		clientAddress = last_msg.substr(loc + strlen(" requested from "));

		std::set<std::string> client_ips;
		get_ips_from_host(clientAddress, client_ips);
		
		for (std::set<std::string>::const_iterator it = client_ips.begin(); it != client_ips.end(); ++it )
		{
			if (std::find(allowed_ips.begin(), allowed_ips.end(), *it) != allowed_ips.end())
				return;  // found the IP in the allowed list of IPs
		}
   	}
	else
	{
		DEBUG_STREAM << "Cannot extract client IP or hostname from tango blackbox, denying access.";
		
		Tango::Except::throw_exception( 
			(const char*) "HOST IP NOT FOUND", 
			(const char*) "host IP not found in tango blackbox", 
			(const char*) "DataManager::always_executed_hook"
			); 
	}
		
	// otherwise, fire exception

   	WARN_STREAM << "Client " << clientAddress.c_str() << " tried to connect but its IP is not in the allowed IP list, denying access." << std::endl;
   	
	Tango::Except::throw_exception( 
		(const char*) "HOST IP NOT ALLOWED", 
		(const char*) "host IP not present in the allowed IP list", 
		(const char*) "DataManager::always_executed_hook"
		); 

}
//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_attr_hardware(vector<long> &attr_list)
{
//	DEBUG_STREAM << "ScriptRunner::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_Output
// 
// description : 	Extract real attribute values for Output acquisition result.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_Output(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "ScriptRunner::read_Output(Tango::Attribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	output_s = const_cast<char *>(output.c_str());
	
	attr.set_value(&output_s, 1);
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_Error
// 
// description : 	Extract real attribute values for Error acquisition result.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_Error(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "ScriptRunner::read_Error(Tango::Attribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	error_s = const_cast<char *>(error.c_str());
	
	attr.set_value(&error_s, 1);
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_ExitStatus
// 
// description : 	Extract real attribute values for ExitStatus acquisition result.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_ExitStatus(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "ScriptRunner::read_ExitStatus(Tango::Attribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	attr.set_value(attr_ExitStatus_read);
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_ScriptsRepositoryPath
// 
// description : 	Extract real attribute values for ScriptsRepositoryPath acquisition result.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_ScriptsRepositoryPath(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "ScriptRunner::read_ScriptsRepositoryPath(Tango::Attribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	scriptsRepositoryPath_s = const_cast<char *>(scriptsRepositoryPath.c_str());
	
	attr.set_value(&scriptsRepositoryPath_s, 1);
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::write_ScriptsRepositoryPath
// 
// description : 	Write ScriptsRepositoryPath attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ScriptRunner::write_ScriptsRepositoryPath(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ScriptRunner::write_ScriptsRepositoryPath(Tango::WAttribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	const Tango::ConstDevString * string_in = NULL;
	attr.get_write_value(string_in);

	string temp(*string_in);
	
	scriptsRepositoryPath = temp;
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::read_CommandLineParams
// 
// description : 	Extract real attribute values for CommandLineParams acquisition result.
//
//-----------------------------------------------------------------------------
void ScriptRunner::read_CommandLineParams(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "ScriptRunner::read_CommandLineParams(Tango::Attribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	size_t num_params = commandLineParams.size();
	
	Tango::DevString * array = new Tango::DevString[num_params];
		
	int i=0;
	for (list<string>::const_iterator it = commandLineParams.begin(); it != commandLineParams.end(); ++it, ++i)
	{
		std::string param_clear(*it);
		std::string param_encrypted = eventually_encrypt_with_im_public_key(param_clear);
		
		array[i] = CORBA::string_dup(param_encrypted.c_str());
	}
		
	attr.set_value(array, num_params, 0, true);
	
	pthread_mutex_unlock(&output_values_mutex);
}

//+----------------------------------------------------------------------------
//
// method : 		ScriptRunner::write_CommandLineParams
// 
// description : 	Write CommandLineParams attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ScriptRunner::write_CommandLineParams(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ScriptRunner::write_CommandLineParams(Tango::WAttribute &attr) entering... "<< endl;
	
	pthread_mutex_lock(&output_values_mutex);
	
	const Tango::ConstDevString * values = NULL;
	attr.get_write_value(values);
	
	int num_values = attr.get_w_dim_x();
	
	commandLineParams.clear();
	for (int i = 0; i < num_values; ++i)
	{
		string param_decrypted;
		bool param_decrypt_ok = eventually_decrypt_with_device_private_key(string(values[i]), param_decrypted);
		if (!param_decrypt_ok)
		{
			pthread_mutex_unlock(&output_values_mutex);
			
			TangoSys_OMemStream os;
			os << "Error, cannot decrypt parameter!";
			safe_set_state_status(Tango::FAULT, os.str());

			return;
		}

		commandLineParams.push_back(param_decrypted);
	}
	
	pthread_mutex_unlock(&output_values_mutex);
}



//+------------------------------------------------------------------
/**
 *	method:	ScriptRunner::run_script
 *
 *	description:	method to execute "RunScript"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void ScriptRunner::run_script(Tango::DevString argin)
{
	DEBUG_STREAM << "ScriptRunner::run_script(): entering... !" << endl;

	//	Add your own code to control device here
	
	string script_name;

	bool script_decrypt_ok = eventually_decrypt_with_device_private_key(std::string(argin), script_name);
	if (!script_decrypt_ok)
	{
		TangoSys_OMemStream os;
		os << "Error, cannot decrypt script name!";
		safe_set_state_status(Tango::FAULT, os.str());

		return;
	}
	
	string script_full_path = scriptsRepositoryPath + "/" + script_name;
	
	char * script_full_path_canonicalized = canonicalize_file_name(script_full_path.c_str());
	if (script_full_path_canonicalized == NULL)
	{
		TangoSys_OMemStream os;
		os << "Error, cannot canonicalize script path \"" << script_full_path.c_str() << "\": " << strerror(errno);
		safe_set_state_status(Tango::FAULT, os.str());

		return;
	}
	
	// no problems found, execute the command
	
	// prepare thread params
	
	pthread_mutex_lock(&output_values_mutex);
	
	thread_params * tp = new thread_params();
	tp->obj = this;
	tp->script = string(script_full_path_canonicalized);
	tp->params = commandLineParams;
	
	pthread_mutex_unlock(&output_values_mutex);
	
	// free canonicalized path
	
	free(script_full_path_canonicalized);
	
	// reset results
	
	pthread_mutex_lock(&output_values_mutex);
	
	output = "";
	error = "";
	exitStatus = -1;
		
	pthread_mutex_unlock(&output_values_mutex);
	
	// set state to RUNNING
	// note: by setting state to RUNNING, all clients will be inhibited in reading/writing attributes by the state machine

	safe_set_state_status(Tango::RUNNING, "Executing command...");

	// join previous thread
	
	if (thread_valid)
		pthread_join(thread, NULL);
	thread_valid = false;
	
	// create and run working thread

	pthread_create(&thread, NULL, ScriptRunner::thread_trampoline, tp);
	thread_valid = true;
	
	// exit... the thread is left running
}

void ScriptRunner::run_script_thread(thread_params * tp)
{
	string script = tp->script;
	list<string> params = tp->params;
	
	delete tp;
	tp = NULL;
	
	// prepare input variables
	
	string script_name = my_basename(script);
	
	char **args = new char *[params.size() + 2];
	
	args[0] = strdup(script_name.c_str());
	int i=1;
	for (list<string>::const_iterator it = params.begin(); it != params.end(); ++it)
	{
		string param = (*it);	
		args[i++] = strdup(param.c_str());
	}
	args[i] = NULL;
	
	// prepare output variables
	
	string scriptError;
	string scriptOutput;
	Tango::DevLong scriptExitStatus;
	
	// run the script
	
	int stdout_pipe[2];
	int stderr_pipe[2];
	
	int stdout_pipe_success = pipe(stdout_pipe);
	if (stdout_pipe_success == -1)
	{
		safe_set_state_status(Tango::FAULT, string("Error, cannot create pipe for stdout: ") + string(strerror(errno)));
		
		for (int j = 0; j < params.size() + 2; ++j)
			if (args[j])
				free(args[j]);
		delete args;
		
		pthread_exit(0);
	}
	
	int stderr_pipe_success = pipe(stderr_pipe);
	if (stderr_pipe_success == -1)
	{
		safe_set_state_status(Tango::FAULT, string("Error, cannot create pipe for stderr: ") + string(strerror(errno)));
		
		close(stdout_pipe[0]);
		close(stdout_pipe[1]);
		
		for (int j = 0; j < params.size() + 2; ++j)
			if (args[j])
				free(args[j]);
		delete args;
		
		pthread_exit(0);
	}
	
	pid_t child_pid = fork();
	if (child_pid == -1)
	{
		safe_set_state_status(Tango::FAULT, string("Error, cannot fork: ") + string(strerror(errno)));
		
		close(stdout_pipe[0]);
		close(stdout_pipe[1]);
		close(stderr_pipe[0]);
		close(stderr_pipe[1]);
		
		for (int j = 0; j < params.size() + 2; ++j)
			if (args[j])
				free(args[j]);
		delete args;
		
		pthread_exit(0);
	}
	
	if (child_pid == 0)
	{
		// the child process
		
		// close the reading side of pipes
		close(stdout_pipe[0]);
		close(stderr_pipe[0]);

		// redirect stdout, stderr to write side of pipes
		if (dup2(stdout_pipe[1], STDOUT_FILENO) == -1)
			_exit(100);
		if (dup2(stderr_pipe[1], STDERR_FILENO) == -1)
			_exit(101);
		
		close(stdout_pipe[1]);
		close(stderr_pipe[1]);
		
		// start the script

		execv(script.c_str(), args);
		
		// execv returned due to some error
		
		_exit(102);
	}
	else
	{
		// the parent process
		
		// close write side of pipes
		close(stdout_pipe[1]);
		close(stderr_pipe[1]);
		
		// read stdout and stderr
		char stdout_buff[2048];
		char stderr_buff[2048];
		while(true)
		{
			int stdout_read_count = read(stdout_pipe[0], stdout_buff, 2048);
			if (stdout_read_count > 0)
				scriptOutput.append(stdout_buff, stdout_read_count);
			
			if (stdout_read_count == -1 && errno == EINTR)
				continue;
			
			int stderr_read_count = read(stderr_pipe[0], stderr_buff, 2048);
			if (stderr_read_count > 0)
				scriptError.append(stderr_buff, stderr_read_count);
		
			if (stderr_read_count == -1 && errno == EINTR)
				continue;
			
			if (stdout_read_count <=0 && stderr_read_count <= 0)
				break;	
		}
		
		// wait for script exit and retrieve exit status
		
		int processExitCode;
		int wait_result = waitpid(child_pid, &processExitCode, 0);
		if (wait_result == -1 || !WIFEXITED(processExitCode))
			scriptExitStatus = 103;
		else
			scriptExitStatus = WEXITSTATUS(processExitCode);
	}
	
	// update exit status
	
	pthread_mutex_lock(&output_values_mutex);
	
	output = eventually_encrypt_with_im_public_key(scriptOutput);
	error = eventually_encrypt_with_im_public_key(scriptError);
	exitStatus = scriptExitStatus;
		
	pthread_mutex_unlock(&output_values_mutex);
	
	// free args array
	
	for (int j = 0; j < params.size() + 2; ++j)
		if (args[j])
			free(args[j]);
	delete args;
	
	// update device state and status 
	
	if (exitStatus == 0)
		safe_set_state_status(Tango::ON, "Script exited with success.");
	else
		safe_set_state_status(Tango::ON, "Script exited with error.");

	// close read side of pipes
	close(stdout_pipe[0]);
	close(stderr_pipe[0]);

	// quit thread
	pthread_exit(0);
}

std::string ScriptRunner::eventually_encrypt_with_im_public_key(const std::string & in)
{
	std::string encrypted;
	//if (botan_utils)
	//	encrypted = botan_utils->encrypt_output_value_with_im_public_key(in);
	//else
		encrypted = in;
	
	DEBUG_STREAM << "Perhaps encrypted with client public key: " << in.c_str() << " -> " << encrypted.c_str();
	
	return encrypted;
}

bool ScriptRunner::eventually_decrypt_with_device_private_key(const std::string & in, std::string & clear)
{
	bool success = false;
	
	//if (botan_utils)
	//{
	//	success = botan_utils->decrypt_input_value_with_device_private_key(in, clear);
	//}
	//else
	//{
		clear = in;
		success = true;
	//}
	
	DEBUG_STREAM << "Perhaps decrypted with device private key: " << in.c_str() << " -> " << clear.c_str();
	
	return success;
}

Tango::DevState ScriptRunner::state()
{
	Tango::DevState state;
	
	pthread_mutex_lock(&state_mutex);
	
	state = TANGO_BASE_CLASS::state();
	
	pthread_mutex_unlock(&state_mutex);
	
	return state;
}

char * ScriptRunner::status()
{
	char * status;
	
	pthread_mutex_lock(&state_mutex);
	
	status = TANGO_BASE_CLASS::status();
	
	pthread_mutex_unlock(&state_mutex);
	
	return status;
}

Tango::DevState ScriptRunner::safe_get_state()
{
	Tango::DevState state;
	
	pthread_mutex_lock(&state_mutex);
	
	state = get_state();
	
	pthread_mutex_unlock(&state_mutex);
	
	return state;
}

string ScriptRunner::safe_get_status()
{
	string status;
	
	pthread_mutex_lock(&state_mutex);
	
	status = get_status();
	
	pthread_mutex_unlock(&state_mutex);
	
	return status;
}

void ScriptRunner::safe_set_state_status(const Tango::DevState & new_state, const string & new_status)
{
	pthread_mutex_lock(&state_mutex);

	set_state(new_state);
	set_status(new_status);
	
	pthread_mutex_unlock(&state_mutex);
	
	DEBUG_STREAM << "New state set: " << Tango::DevStateName[new_state] <<  " with status: " << new_status.c_str() << endl;
}

void *ScriptRunner::thread_trampoline (void *param)
{
	static_cast<thread_params *>(param)->obj->run_script_thread(static_cast<thread_params *>(param));

	return NULL;
}

void ScriptRunner::get_ips_from_host(const std::string & node, std::set<std::string> & ips)
{
	struct addrinfo * results = NULL;
	getaddrinfo(node.c_str(), NULL, NULL, &results);
	
	struct addrinfo * item = results;
	while(item)
	{
		std::string ip_string;
		char buffer[1024];
		
		if (item->ai_family == PF_INET)
		{              
			struct sockaddr_in *addr = (struct sockaddr_in *) item->ai_addr;     
                 
			ip_string.assign(inet_ntop(addr->sin_family, &addr->sin_addr, buffer, sizeof(buffer)));
			
			ips.insert(ip_string);
			
			//DEBUG_STREAM << "Found IP for host: " << node.c_str() << " ip: " << ip_string.c_str() << std::endl;
		}
		else if (item->ai_family == PF_INET6)
		{     
			struct sockaddr_in6 * addr6 = (struct sockaddr_in6 *) item->ai_addr;  
              
			ip_string.assign(inet_ntop(addr6->sin6_family, &addr6->sin6_addr, buffer, sizeof(buffer)));
			
			ips.insert(ip_string);
			
			//DEBUG_STREAM << "Found IP for host: " << node.c_str() << " ip: " << ip_string.c_str() << std::endl;
		}
		
		item = item->ai_next;
	}
	
	freeaddrinfo(results);
}


}	//	namespace
